"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Visitor_1 = __importDefault(require("@swc/core/Visitor"));
const DEFAULT_OPTS = {
    preventFullImport: false,
    skipDefaultConversion: false,
    style: false,
};
function barf(msg) {
    throw new Error('swc-plugin-transform-imports: ' + msg);
}
function transformImportPath(transformOption, importName) {
    if (!transformOption)
        return;
    const isFunction = typeof transformOption === 'function';
    // If transformOption is a function or defined in a JS file.
    if (/\.js$/i.test(transformOption) && !/[\$\{\}]/.test(transformOption) || isFunction) {
        let transformFn;
        try {
            transformFn = isFunction ? transformOption : require(transformOption);
        }
        catch (error) {
            barf('failed to require transform file ' + transformOption);
        }
        if (typeof transformFn !== 'function') {
            barf('expected transform function to be exported from ' + transformOption);
        }
        return transformFn(importName);
    }
    const transformedImport = transformOption.replace(/\$\{\s?member\s?\}/ig, importName);
    return transformedImport;
}
class PluginTransformImport extends Visitor_1.default {
    constructor(opts = {}) {
        super();
        this.opts = {};
        // Add default options to the every opts object.
        for (const key of Object.keys(opts)) {
            this.opts[key] = Object.assign(Object.assign({}, DEFAULT_OPTS), opts[key]);
        }
    }
    visitModuleItems(nodes) {
        var _a, _b;
        const { opts } = this;
        const transformedNodes = [];
        // Return if options are not provided.
        if (Object.keys(opts).length === 0) {
            return nodes;
        }
        for (const node of nodes) {
            const { type } = node;
            const isValidTranformCandidate = type === 'ImportDeclaration' && (node.source.value in opts);
            if (!isValidTranformCandidate) {
                transformedNodes.push(node);
                continue;
            }
            // node has properties 'source' and 'specifiers' attached.
            // node.source is the library/module name, aka 'react-bootstrap'.
            // node.specifiers is an array of ImportSpecifier | ImportDefaultSpecifier | ImportNamespaceSpecifier
            const { source, specifiers } = node;
            const { preventFullImport, skipDefaultConversion, transform, style } = opts[source.value];
            const isDefaultImportExist = specifiers.some(specifier => specifier.type === 'ImportDefaultSpecifier');
            //      import * as name from 'module'; (ImportNamespaceSpecifier)
            //      import name from 'module'; (ImportDefaultSpecifier)
            if (isDefaultImportExist && preventFullImport) {
                barf('Import of entire module ' + source.value + ' not allowed due to preventFullImport setting');
            }
            // Going through each specifier and transforming the import path.
            // aka Row, Grid, { Grid as MyGrid}..
            for (const specifier of specifiers) {
                const { type, local } = specifier;
                const doesImportedValueExist = type === 'ImportSpecifier' && ((_a = specifier === null || specifier === void 0 ? void 0 : specifier.imported) === null || _a === void 0 ? void 0 : _a.value);
                const actualImportVariable = doesImportedValueExist ? (_b = specifier === null || specifier === void 0 ? void 0 : specifier.imported) === null || _b === void 0 ? void 0 : _b.value : local.value;
                const shouldSkipDefaultConversion = skipDefaultConversion && doesImportedValueExist;
                // Swap out the import with one that doesn't include member imports.
                // Member imports should each get their own import line
                // transform this:
                //      import Bootstrap, { Grid } from 'react-bootstrap';
                // into this:
                //      import Bootstrap from 'react-bootstrap';
                //      import Grid from 'react-bootstrap/lib/Grid';
                // Create a new Import Declaration node for each named import
                if (type === 'ImportSpecifier') {
                    const newSpecifier = Object.assign(Object.assign({}, specifier), (shouldSkipDefaultConversion ? {} : {
                        imported: null,
                        type: "ImportDefaultSpecifier",
                    }));
                    const value = transformImportPath(transform, actualImportVariable || '');
                    const copyNode = Object.assign(Object.assign({}, node), { source: Object.assign(Object.assign({}, source), { value }), specifiers: [newSpecifier], type: "ImportDeclaration" });
                    transformedNodes.push(copyNode);
                    if (style) {
                        const styleNode = Object.assign(Object.assign({}, node), { source: Object.assign(Object.assign({}, source), { value: `${value}/style` }), specifiers: [], type: "ImportDeclaration" });
                        transformedNodes.push(styleNode);
                    }
                }
                else if (type === 'ImportDefaultSpecifier') {
                    const nameImportsFilteredNode = Object.assign(Object.assign({}, node), { specifiers: [specifier] });
                    transformedNodes.push(nameImportsFilteredNode);
                }
            }
        }
        // If import declaration is a candidate for transformation
        // We push all the new import declarations to the transformedNodes array.
        // Else we push the original node to the transformedNodes array.
        return transformedNodes;
    }
}
exports.default = PluginTransformImport;
